//          _/             _/_/
//        _/_/           _/_/_/
//      _/_/_/_/         _/_/_/
//      _/_/_/_/_/       _/_/_/              ____________________________________________ 
//      _/_/_/_/_/       _/_/_/             /                                           / 
//      _/_/_/_/_/       _/_/_/            /                                   M25P128 / 
//      _/_/_/_/_/       _/_/_/           /                                           /  
//      _/_/_/_/_/_/     _/_/_/          /                                   128Mbit / 
//      _/_/_/_/_/_/     _/_/_/         /                              SERIAL FLASH / 
//      _/_/_/ _/_/_/    _/_/_/        /                                           / 
//      _/_/_/  _/_/_/   _/_/_/       /                  Verilog Behavioral Model / 
//      _/_/_/   _/_/_/  _/_/_/      /                               Version 1.0 / 
//      _/_/_/    _/_/_/ _/_/_/     /                                           /
//      _/_/_/     _/_/_/_/_/_/    /           Copyright (c) 2008 Numonyx B.V. / 
//      _/_/_/      _/_/_/_/_/    /___________________________________________/ 
//      _/_/_/       _/_/_/_/      
//      _/_/          _/_/_/  
// 
//     
//             NUMONYX              



/*-------------------------------------------------------------------------
-- The procedures here following may be used to send
-- commands to the serial flash.
-- These procedures must be combined using one of the following sequences:
--  
-- 1) send_command / close_comm 
-- 2) send_command / send_address / close_comm
-- 3) send_command / send_address / send_data /close_comm
-- 4) send_command / send_address / read / close_comm
-- 5) send_command / read / close_comm
-------------------------------------------------------------------------*/



module StimTasks (S, HOLD, D, Vcc, Vpp_W, clock_active); //P128



    `include "include/DevParam.h"

    output S; 
    reg S;
    
    output [`VoltageRange] Vcc;
    reg [`VoltageRange] Vcc;

    output clock_active;
    reg clock_active;

    `ifdef DUAL
        output DQ0, DQ1; 
        reg DQ0='bZ; reg DQ1='bZ; 
    `else    
        output D; 
        reg D='bZ; 
    `endif

    `ifdef Vpp_pin
        output [`VoltageRange] Vpp; reg [`VoltageRange] Vpp; `endif

    `ifdef Vpp_W_pin
        output [`VoltageRange] Vpp_W; reg [`VoltageRange] Vpp_W; `endif    
    
    `ifdef HOLD_pin
        output HOLD; reg HOLD; `endif
    
    `ifdef W_pin
        output W; reg W; `endif

    `ifdef RESET_pin
        output RESET; reg RESET; `endif





    //-----------------
    // Initialization
    //-----------------


    task init;
    begin
        
        S = 1;
        `ifdef HOLD_pin
          HOLD = 1; `endif
        `ifdef RESET_pin
          RESET = 1; `endif
        `ifdef W_pin
          W = 1; `endif  
        power_up;

    end
    endtask



    task power_up;
    begin

        Vcc='d3000;
        `ifdef Vpp_pin
          Vpp='d0; `endif
        `ifdef Vpp_W_pin
          Vpp_W='d3000; `endif  
        #(write_access_power_up_delay+100);

    end
    endtask







    //----------------------------------------------------------
    // Tasks for send commands, send adressses, and read memory
    //----------------------------------------------------------


    task send_command;

    input [cmdDim-1:0] cmd;
    
    integer i;
    
    begin

        clock_active = 1;  #(T/4);
        S=0; #(T/4); 

        for (i=cmdDim-1; i>=1; i=i-1) begin
            `D=cmd[i]; #T;
        end

        `D=cmd[0]; #(T/2+T/4); 

    end
    endtask







    task send_address;

    input [addrDim-1 : 0] addr;

    integer i;
    
    begin

        #(T/4);

        // first MSB bits are don't care
        `ifdef M25PE16
          `D=0; #T; `D=0;#T; `D=0; #T;
        `elsif M25PX32
           `D=0; #T; `D=0;#T;
        `endif   

        for (i=addrDim-1; i>=1; i=i-1) begin
            `D = addr[i]; #T;
        end    

        `D = addr[0];  #(T/2+T/4);

    end
    endtask 








    task send_data;

    input [dataDim-1:0] data;
    
    integer i;
    
    begin

        #(T/4);

        for (i=dataDim-1; i>=1; i=i-1) begin
            `D=data[i]; #T;
        end

        `D=data[0]; #(T/2+T/4); 

    end
    endtask





    `ifdef DUAL
    
        task send_data_dual;

        input [dataDim-1:0] data;
        
        integer i;
        
        begin

            #(T/4);

            for (i=dataDim-1; i>=3; i=i-2) begin
                `Q=data[i]; 
                `D=data[i-1];
                #T;
            end

            `Q=data[1];
            `D=data[0]; #(T/2+T/4);
            `Q=1'bZ;

        end
        endtask

    `endif    






    task read;

    input n;
    integer n, i;

    for (i=1; i<=n; i=i+1) begin 
        #(8*T);
    end  

    endtask



    
    task read_dual;

    input n;
    integer n, i;

    begin
    `D = 1'bZ;
    for (i=1; i<=n; i=i+1) begin 
        #(4*T);
    end   
    end
    endtask





    // shall be used in a sequence including send command
    // and close communication tasks
    
    task add_clock_cycle;

    input integer n;
    integer i;

        for(i=1; i<=n; i=i+1) #T;

    endtask







    task close_comm;

    begin

        S = 1;
        clock_active = 0;
        # T;
        # 100;

    end
    endtask





    //------------------
    // others tasks
    //------------------


    `ifdef HOLD_pin
    task set_HOLD;
    input value;
        HOLD = value;
    endtask
    `endif


    task set_clock;
    input value;
        clock_active = value;
    endtask 


    task set_S;
    input value;
        S = value;
    endtask
    

    task setVcc;
    input [`VoltageRange] value;
        Vcc = value;
    endtask


    task Vcc_waveform;
    input [`VoltageRange] V1; input time t1;
    input [`VoltageRange] V2; input time t2;
    input [`VoltageRange] V3; input time t3;
    begin
      Vcc=V1; #t1;
      Vcc=V2; #t2;
      Vcc=V3; #t3;
    end
    endtask


    `ifdef Vpp_pin
    task setVpp;
    input [`VoltageRange] value;
        Vpp = value;
    endtask
    `endif

    `ifdef Vpp_W_pin
      task set_Vpp_W;
      input [`VoltageRange] value;
          Vpp_W = value;
      endtask
    `endif

    `ifdef W_pin
    task set_W;
    input value;
        W = value;
    endtask
    `endif



    `ifdef RESET_pin
    
    task set_RESET;
    input value;
        RESET = value;
    endtask

    task RESET_pulse;
    begin
        RESET = 0;
        #100;
        RESET = 1;
    end
    endtask
    
    `endif

    




    //------------------------------------------
    // Tasks to send complete command sequences
    //------------------------------------------


    task write_enable;
    begin
        send_command('h06); //write enable
        close_comm;
        #100;
    end  
    endtask


    task unlock_sector;
    input [addrDim-1:0] A;
    begin
        // write lock register (to unlock sector to be programmed)
        tasks.write_enable;
        tasks.send_command('hE5);
        tasks.send_address(A);
        tasks.send_data('h00);
        tasks.close_comm;
        #100;
    end 
    endtask





endmodule    
