`timescale 1ns/1ns

//this module don't take into account the power supply nor the geographical
//addressing of the various slots. Those connections if required must be
//implemented for each module.
//The same is true for U.D. pins on P2 and all the P0 connections.

module VmeMaster
#(
    parameter   gfd_FileOutput = 32'h8000_0001, //default is standard out
                g_VerboseAccesses = 1'b1 //write access results to standard out
)
(
    output reg Trst_o,
    output reg Tck_o,
    input Tdo_i,
    output reg Tdi_o,
    output Tms_o,
    inout [31:0] D_b,
    output reg SysClk_ok,
    output reg Ds1_o,
    output reg Ds2_o,
    output reg Write_o,
    input DtAck_i,
    output reg As_o,
    output reg IAck_o, 
    output reg [5:0] Am_o,
    inout [31:1] A_b,
    input [7:1] Irq_i,
    input Retry_i,
    input Berr_i,
    output reg SysReset_o,
    inout LWord_b);

initial begin
    Ds2_o       = 1'b1; 
    Ds1_o       = 1'b1;
    As_o        = 1'b1;
    IAck_o      = 1'b1;
    SysReset_o  = 1'b1;
end

reg [31:0] a_D = 32'hz;
reg [31:1] a_A = 31'hz;
reg a_LWord = 1'bz;

integer fd_Output;
reg fd_FileOutput;
reg VerboseAccesses;

initial fd_FileOutput = gfd_FileOutput;
initial VerboseAccesses= g_VerboseAccesses;

assign D_b      = a_D;
assign A_b      = a_A;
assign LWord_b  = a_LWord;

//**********************************
//VME write cycle
//**********************************
task SimpleWrite;
    input [31:0] Address; 
    input [5:0] AddressModifier;
    input [31:0] Data;
    output [7:0] ExitCode; // 0=Done, 1=Timeout, 2=BusError, 3=Retry
integer i;		
begin
    a_A         = Address[31:1];
    a_LWord     = 1'b0;
    Am_o 	= AddressModifier;
    a_D 	= Data;
    Write_o	= 1'b0;
    # 10;
    As_o	= 1'b0;
    #10;
    Ds1_o       = 1'b0;
    Ds2_o       = 1'b0;
    i=0;
    while (DtAck_i && Retry_i && Berr_i && ~(i==1000)) #10 i=i+1; 
    if (~Berr_i) begin
	    if (VerboseAccesses) $display(fd_FileOutput,"Bus Error asserted");
	    ExitCode    = 8'h2;
	    As_o	    = 1'b1;
	    Ds1_o       = 1'b1;
	    Ds2_o       = 1'b1;
	    a_D	        = 32'hz;
        a_A         = 31'hz;
    end else if (~Retry_i) begin
	    if (VerboseAccesses) $display(fd_FileOutput,"Slave asserted retry request");
	    ExitCode    = 8'h3;
	    As_o	    = 1'b1;
	    Ds1_o       = 1'b1;
	    Ds2_o       = 1'b1;
	    a_D	        = 32'hz;
        a_A         = 31'hz;
    end else if (DtAck_i) begin
	    if (VerboseAccesses) $display(fd_FileOutput,"No module response to the vme write request: time out");
	    ExitCode    = 8'h1;
	    As_o	    = 1'b1;
	    Ds1_o       = 1'b1;
	    Ds2_o       = 1'b1;
	    a_D	        = 32'hz;
        a_A         = 31'hz;
    end else begin		
        # 10; 
        As_o        = 1'b1;
        Ds1_o       = 1'b1;
        Ds2_o       = 1'b1;
	    a_D         = 32'hz;
        a_A         = 31'hz;
	    i=0;
	    while (~DtAck_i && ~(i==1000)) #10 i=i+1;	
	    if (~DtAck_i) begin
	        if (VerboseAccesses) $display(fd_FileOutput,"Unable to close the vme write in a usefull time: time out");
     	    ExitCode    = 8'h1;                    	
        end else begin 
            ExitCode =8'h0;
	        if (VerboseAccesses) $display(fd_FileOutput,"VME write cycle: d = %h at address %h address at %t",Data ,Address, $time);            
        end
    end
    a_A         = 31'bz;
    a_LWord     = 1'bz;
    a_D     	= 32'bz;
end
endtask

//**********************************
//VME read cycle
//**********************************

task SimpleRead;
    input [31:0] Address; 
    input [5:0] AddressModifier;
    output [31:0] DataRead;  
    output [7:0] ExitCode; // 0=Done, 1=Timeout, 2=BusError, 3=Retry    
integer i;	
begin
    a_A         = Address[31:1];
    a_D         = 32'hz;
    a_LWord	    = 1'b0;
    Am_o        = AddressModifier;
    Write_o     = 1'b1;
    # 10;
    As_o	    = 1'b0;
    #10;
    Ds1_o       = 1'b0;
    Ds2_o       = 1'b0;
    i=0;
    while (DtAck_i && ~(i==1000)) #10 i=i+1;	
    if (DtAck_i) begin
        if (VerboseAccesses) $display(fd_FileOutput,"No module response to the vme read request @ 32'h%h : time out", Address);
        ExitCode    = 8'd1;
    	As_o	    = 1'b1;
    	Ds1_o       = 1'b1;
    	Ds2_o       = 1'b1;
    end else begin		
	    DataRead    = D_b;
	    # 10; 
	    As_o	    = 1'b1;
	    Ds1_o       = 1'b1;
	    Ds2_o       = 1'b1;
	    i=0;
	    while (~DtAck_i && ~(i==1000)) #10 i=i+1;	
	    if (~DtAck_i) begin
            if (VerboseAccesses) $display(fd_FileOutput,"Unable to close the vme read in a usefull time: time out");
            ExitCode = 8'd2;
        end else begin
             if (VerboseAccesses) $display(fd_FileOutput,"VME read cycle: d = %h at address %h of A32 address space at %t", DataRead, Address,$time);
             ExitCode   = 8'd0;
	    end
    end
    a_A         = 31'bz;
    a_LWord     = 1'bz;
    a_D     	= 32'bz;
end
endtask

//***************************************
// WAIT FOR INTERRUPT
//***************************************

task AcknowledgeInterrupt;
    output  [2:0] InterruptLevel;
    output  [31:0] interrupt_vector;
    output  ExitCode; // 0=Interrupt received, 1=Time out, 2=Bus Error during the acknowledge     
integer i;
begin
	if (VerboseAccesses) $display(fd_FileOutput,"Waiting for interrupt.....");
    InterruptLevel = 'h0;
    a_D 	= 32'bz;
    i	= 0;
    while (&Irq_i && ~(i==150)) #100 i=i+1;	
    if (&Irq_i) begin
	   	if (VerboseAccesses)  $display(fd_FileOutput,"No interrupt received: time out");
	    interrupt_vector	= 32'hx;
        ExitCode = 'd1;
    end else begin
	    IAck_o	= 1'b0;    
        a_A[3:1]	= ~Irq_i[7] ? 3'h7 : (~Irq_i[6] ? 3'h6 : (~Irq_i[5] ? 3'h5 : (~Irq_i[4] ? 3'h4 : (~Irq_i[3] ? 3'h3 : (~Irq_i[2] ? 3'h2 : ~Irq_i[1] ? 3'h1 : 3'hx)))));
        #10 As_o = 1'b0; 
        InterruptLevel = a_A[3:1];
        if (VerboseAccesses) $display(fd_FileOutput,"Received interrupt reqest of priority level %d", InterruptLevel);
        Ds1_o	= 1'b0;      
        Write_o		= 1'b1;
        i = 0;
        while (DtAck_i && ~(i==1000)) #1 i=i+1;
        if (DtAck_i) begin
            	if (VerboseAccesses) $display(fd_FileOutput,"No module responded to the interrupt acknoledge in a usefull time");
            ExitCode = 'd2;
	    end else begin
	        interrupt_vector	= D_b ;
            	if (VerboseAccesses) $display(fd_FileOutput,"Received interrupt vector: %h", 	interrupt_vector);
            ExitCode = 'd0;
	    end
        #10;
    	As_o	= 1'b1;
    	Ds1_o	= 1'b1;  
    	IAck_o 	= 1'b1;   
    	i=0;
    	while (~DtAck_i && ~(i==10)) #1000 i=i+1;	
    	if (~DtAck_i) begin
            	if (VerboseAccesses) $display(fd_FileOutput,"Unable to close the vme interrupt acknoledge in a usefull time: time out");
            ExitCode = 'd2;
        end
    end
    a_A         = 31'bz;
    a_LWord     = 1'bz;
    a_D     	= 32'bz;
end 
endtask  

//***********************************
// StdIn prompt
//***********************************

integer fd_MyFifoIn, fd_MyFifoOut;
initial begin
    fd_MyFifoIn = $fopen("Py2V.fifo", "r");
end

task VmePrompt;
reg [31:0] address32, data32;
reg [2:0] IntLevel3;
reg [7:0] command;
integer outcode;
begin
//	$write("Enter a command [D (start to Dump the simulation), A (wAit), W (Write), R (Read), I (wait for an Interrupt), S (Stop)]: ");
//	command=$fgetc(32'h8000_0000);
//	outcode=$fgetc(32'h8000_0000); //just to get rid of the nl
  command=$fgetc(fd_MyFifoIn);
  if (command!=8'hff) begin
	outcode=$fgetc(fd_MyFifoIn); //just to get rid of the nl 	
    if (command=="S" || command=="s") begin
        $fclose(fd_MyFifoIn);
        $finish;
	end else if (command=="D" || command=="d") begin
	    $dumpfile("simout.vcd");
	    $dumpvars(0, tb_VFC);			
	end else if (command=="I" || command=="i") begin		
//      $display(fd_FileOutput,"Which level of interrupt(1 to 7)?  0x");
        AcknowledgeInterrupt(IntLevel3, data32, command);
        fd_MyFifoOut = $fopen("V2Py.fifo", "w");
        $fwrite(fd_MyFifoOut, "%x\n", command);
        $fwrite(fd_MyFifoOut, "%x\n", IntLevel3);
        $fwrite(fd_MyFifoOut, "%x\n", data32);
        $fclose(fd_MyFifoOut);  
	end else if (command=="A" || command=="a") begin
//		$display(fd_FileOutput,"For how long do you want to wait?  ");
//		outcode=$fscanf(32'h8000_0000, "%d", data32);
//		outcode=$fgetc(32'h8000_0000); //just to get rid of the nl	
		outcode=$fscanf(fd_MyFifoIn, "%d", data32);
		outcode=$fgetc(fd_MyFifoIn); //just to get rid of the nl 	
       		# data32;
	end else if (command=="X" || command=="x") begin
		SysReset_o = 1'b0;
		#200;
		SysReset_o = 1'b1;
		#200;
	end else if (command=="W" || command=="w") begin
//		$display(fd_FileOutput,"At which address you want to write?");
//		outcode=$fscanf(32'h8000_0000, "%h", address32);
//		outcode=$fgetc(32'h8000_0000); //just to get rid of the nl		
		outcode=$fscanf(fd_MyFifoIn, "%h", address32);
		outcode=$fgetc(fd_MyFifoIn); //just to get rid of the nl 
//      $display(fd_FileOutput,"What do you want to write?");
//		outcode=$fscanf(32'h8000_0000, "%h", data32);
//		outcode=$fgetc(32'h8000_0000); //just to get rid of the nl	
		outcode=$fscanf(fd_MyFifoIn, "%h", data32);
		outcode=$fgetc(fd_MyFifoIn); //just to get rid of the nl 		
        SimpleWrite(address32, 6'h09, data32, command);
//      $display(fd_FileOutput,"Executed with exit code %d", command);
        fd_MyFifoOut = $fopen("V2Py.fifo", "w");
        $fwrite(fd_MyFifoOut, "%x\n", command);
        $fclose(fd_MyFifoOut); 	
    end else if (command=="R" || command=="r") begin
//   	$display(fd_FileOutput,"At which address you want to read?");
//		outcode=$fscanf(32'h8000_0000, "%h", address32);
//		outcode=$fgetc(32'h8000_0000); //just to get rid of the nl		
		outcode=$fscanf(fd_MyFifoIn, "%h", address32);
		outcode=$fgetc(fd_MyFifoIn); //just to get rid of the nl 
        SimpleRead(address32, 6'h09, data32, command);
//      $display(fd_FileOutput,"Executed with exit code %d", command);
//      $display(fd_FileOutput,"Read %x", data32);        
        fd_MyFifoOut = $fopen("V2Py.fifo", "w");
        $fwrite(fd_MyFifoOut, "%x\n", command);
        $fwrite(fd_MyFifoOut, "%x\n", data32);
        $fclose(fd_MyFifoOut);
    end else 	$display(fd_FileOutput,"%tns: %c is an UNKNOWN COMMAND (hex is %x)", $time, command, command);
  end	
end
endtask

always begin
VmePrompt();
#1000;
end

/*
initial begin
SysReset_o = 1'b1;
#5;
SysReset_o = 1'b0;
#50;
SysReset_o = 1'b1;
#200;
end
*/
endmodule
